% PURPOSE : Demonstrate the differences between the following
% filters on a simple DBN.
%           
%           3) Particle Filter                 (PF)
%           4) PF with Rao Blackwellisation    (RBPF)



% For more details refer to:

% AUTHORS  : Nando de Freitas      (jfgf@cs.berkeley.edu)
%            Arnaud Doucet         (ad2@eng.cam.ac.uk)
% DATE     : 9 May 2000

clear all;
echo off;

% INITIALISATION AND PARAMETERS:
% ==============================

doPlot = 0;                 % 1 plot online. 0 = only plot at the end.
T = 50;                     % Number of time steps.
N = 100;                    % Number of particles.
resamplingScheme = 1;       % The possible choices are
                            % systematic sampling (2),
                            % residual (1)
                            % and multinomial (3). 
                            % They're all O(N) algorithms. 

% DEFINE THE TRANSITION, OBSERVATION AND INITIAL MATRICES:
% =======================================================

% Specificy the transition matrices Tr, the observation matrices
% Oa, Ob and Oc and the initial distribution P0.
veryNoisy = 0; % Have the choice of two possible experiments.
if veryNoisy
  trb = [.5 .5 
         .5 .5];
  tra = cell(2,1);
  trc = cell(2,1);
  tra{1} = [.5 .5    %if B(t-1)=0; 
            .4 .6];
  tra{2} = [.2 .8    %if B(t-1)=1; 
            .2 .8];
  trc{1} = [.5 .5    %if B(t-1)=0; 
            .4 .6];
  trc{2} = [.4 .6    %if B(t-1)=1; 
            .7 .3];
  Oa = [.5 .5
        .3 .7];
  Ob = [.5 .5 
        .3 .7];
  Oc = [.5 .5
        .3 .7];
  Pa0 = [.5 .5]'; 
  Pb0 = [.6 .4]';
  Pc0 = [.3 .7]';
else
  trb = [.2 .8 
         .2 .8];
  tra{1} = [.9 0.1    %if B(t-1)=0; 
            .9 0.1];
  tra{2} = [.8 0.2    %if B(t-1)=1; 
            .8 0.2];
  trc{1} = [0.9 .1    %if B(t-1)=0; 
            0.9 .1];
  trc{2} = [0.9 .1    %if B(t-1)=1; 
            0.9 .1];
  Oa = [.1 .9
        .1 .9];
  Ob = [.1 .9
        .1 .9];
  Oc = [.1 .9
        .1 .9];
  Pa0 = [1 0]';
  Pb0 = [0 1]';
  Pc0 = [0 1]';
end;

% GENERATE THE DATA:
% ==================

a = zeros(1,T);
b = zeros(1,T);
c = zeros(1,T);
ya = zeros(1,T);
yb = zeros(1,T);
yc = zeros(1,T);

a(1) = rand > Pa0(1,1);
b(1) = rand > Pb0(1,1);
c(1) = rand > Pc0(1,1);

if a(1) == 0,
  ya(1) = rand > Oa(1,1);
else
  ya(1) = rand > Oa(2,1);
end;
if b(1) == 0,
  yb(1) = rand > Ob(1,1);
else
  yb(1) = rand > Ob(2,1);
end;
if c(1) == 0,
  yc(1) = rand > Oc(1,1);
else
  yc(1) = rand > Oc(2,1);
end;

for t=2:T,
  % Sample the transitions.
  if b(t-1) == 0
    b(t) = rand > trb(1,1);
    if a(t-1) == 0
      a(t) = rand > tra{1}(1,1); 
    else
      a(t) = rand > tra{1}(2,1); 
    end;
    if c(t-1) == 0
      c(t) = rand > trc{1}(1,1); 
    else
      c(t) = rand > trc{1}(2,1); 
    end;    
  else
    if a(t-1) == 0
      a(t) = rand > tra{2}(1,1); 
    else
      a(t) = rand > tra{2}(2,1); 
    end;
    if c(t-1) == 0
      c(t) = rand > trc{2}(1,1); 
    else
      c(t) = rand > trc{2}(2,1); 
    end; 
    b(t) = rand > trb(2,1);
  end;  
  % Sample the observations.
  if a(t) == 0,
    ya(t) = rand > Oa(1,1);
  else
    ya(t) = rand > Oa(2,1);
  end;
  if b(t) == 0,
    yb(t) = rand > Ob(1,1);
  else
    yb(t) = rand > Ob(2,1);
  end;
  if c(t) == 0,
    yc(t) = rand > Oc(1,1);
  else
    yc(t) = rand > Oc(2,1);
  end;
end;


% PLOT THE GENERATED DATA:
% ========================
figure(1)
clf;
subplot(311);
plot(1:T,a,'r');
ylabel('A_t','fontsize',15);
axis([0 T -.1 1.1]);
subplot(312);
plot(1:T,b,'r');
ylabel('B_t','fontsize',15);
axis([0 T -.1 1.1]);
subplot(313);
plot(1:T,c,'r');
ylabel('C_t','fontsize',15);
axis([0 T -.1 1.1]);
xlabel('Time (t)','fontsize',15);

fprintf('\n')
fprintf('Press a key to continue')  
%pause;
fprintf('\n')
fprintf('\n')
fprintf('Training has started')
fprintf('\n')


% COMPUTE FILTERING DISTRIBUTION WITH EXACT HMM FILTER:
% =====================================================

% 0- Initialisation:
predDist = zeros(8,T);
filtTmp  = zeros(8,T);
filtDist = zeros(8,T); % for a,b,c = 000, 001, 010, 011, 
                       %             100, 101, 110, 111
filtDist(1,1) = Pa0(1,1)*Pb0(1,1)*Pc0(1,1);    % 000
filtDist(2,1) = Pa0(1,1)*Pb0(1,1)*Pc0(2,1);    % 001
filtDist(3,1) = Pa0(1,1)*Pb0(2,1)*Pc0(1,1);    % 010
filtDist(4,1) = Pa0(1,1)*Pb0(2,1)*Pc0(2,1);    % 011
filtDist(5,1) = Pa0(2,1)*Pb0(1,1)*Pc0(1,1);    % 100
filtDist(6,1) = Pa0(2,1)*Pb0(1,1)*Pc0(2,1);    % 101
filtDist(7,1) = Pa0(2,1)*Pb0(2,1)*Pc0(1,1);    % 110
filtDist(8,1) = Pa0(2,1)*Pb0(2,1)*Pc0(2,1);    % 111
tic;
flops(0);
for t=2:T,
  % 1- Prediction step:
  predDist(1,t) = trb(1,1)*( tra{1}(1,1) *( trc{1}(1,1) * filtDist(1,t-1) ...
					  + trc{1}(2,1) * filtDist(2,t-1) ) ...			     
                            +tra{1}(2,1) *( trc{1}(1,1) * filtDist(5,t-1) ...
					  + trc{1}(2,1) * filtDist(6,t-1) )) ...		
                 +trb(2,1)*( tra{2}(1,1) *( trc{2}(1,1) * filtDist(3,t-1) ...
					  + trc{2}(2,1) * filtDist(4,t-1) ) ...		
                            +tra{2}(2,1) *( trc{2}(1,1) * filtDist(7,t-1) ...
					  + trc{2}(2,1) * filtDist(8,t-1) ));
  predDist(2,t) = trb(1,1)*( tra{1}(1,1) *( trc{1}(1,2) * filtDist(1,t-1) ...
					  + trc{1}(2,2) * filtDist(2,t-1) ) ...			     
                            +tra{1}(2,1) *( trc{1}(1,2) * filtDist(5,t-1) ...
					  + trc{1}(2,2) * filtDist(6,t-1) )) ...		
                 +trb(2,1)*( tra{2}(1,1) *( trc{2}(1,2) * filtDist(3,t-1) ...
					  + trc{2}(2,2) * filtDist(4,t-1) ) ...		
                            +tra{2}(2,1) *( trc{2}(1,2) * filtDist(7,t-1) ...
					  + trc{2}(2,2) * filtDist(8,t-1) ));
  predDist(3,t) = trb(1,2)*( tra{1}(1,1) *( trc{1}(1,1) * filtDist(1,t-1) ...
					  + trc{1}(2,1) * filtDist(2,t-1) ) ...			     
                            +tra{1}(2,1) *( trc{1}(1,1) * filtDist(5,t-1) ...
					  + trc{1}(2,1) * filtDist(6,t-1) )) ...		
                 +trb(2,2)*( tra{2}(1,1) *( trc{2}(1,1) * filtDist(3,t-1) ...
					  + trc{2}(2,1) * filtDist(4,t-1) ) ...		
                            +tra{2}(2,1) *( trc{2}(1,1) * filtDist(7,t-1) ...
					  + trc{2}(2,1) * filtDist(8,t-1) ));
  predDist(4,t) = trb(1,2)*( tra{1}(1,1) *( trc{1}(1,2) * filtDist(1,t-1) ...
					  + trc{1}(2,2) * filtDist(2,t-1) ) ...			     
                            +tra{1}(2,1) *( trc{1}(1,2) * filtDist(5,t-1) ...
					  + trc{1}(2,2) * filtDist(6,t-1) )) ...		
                 +trb(2,2)*( tra{2}(1,1) *( trc{2}(1,2) * filtDist(3,t-1) ...
					  + trc{2}(2,2) * filtDist(4,t-1) ) ...		
                            +tra{2}(2,1) *( trc{2}(1,2) * filtDist(7,t-1) ...
					  + trc{2}(2,2) * filtDist(8,t-1) ));
  predDist(5,t) = trb(1,1)*( tra{1}(1,2) *( trc{1}(1,1) * filtDist(1,t-1) ...
					  + trc{1}(2,1) * filtDist(2,t-1) ) ...			     
                            +tra{1}(2,2) *( trc{1}(1,1) * filtDist(5,t-1) ...
					  + trc{1}(2,1) * filtDist(6,t-1) )) ...		
                 +trb(2,1)*( tra{2}(1,2) *( trc{2}(1,1) * filtDist(3,t-1) ...
					  + trc{2}(2,1) * filtDist(4,t-1) ) ...		
                            +tra{2}(2,2) *( trc{2}(1,1) * filtDist(7,t-1) ...
					  + trc{2}(2,1) * filtDist(8,t-1) ));
  predDist(6,t) = trb(1,1)*( tra{1}(1,2) *( trc{1}(1,2) * filtDist(1,t-1) ...
					  + trc{1}(2,2) * filtDist(2,t-1) ) ...			     
                            +tra{1}(2,2) *( trc{1}(1,2) * filtDist(5,t-1) ...
					  + trc{1}(2,2) * filtDist(6,t-1) )) ...		
                 +trb(2,1)*( tra{2}(1,2) *( trc{2}(1,2) * filtDist(3,t-1) ...
					  + trc{2}(2,2) * filtDist(4,t-1) ) ...		
                            +tra{2}(2,2) *( trc{2}(1,2) * filtDist(7,t-1) ...
					  + trc{2}(2,2) * filtDist(8,t-1) ));
  predDist(7,t) = trb(1,2)*( tra{1}(1,2) *( trc{1}(1,1) * filtDist(1,t-1) ...
					  + trc{1}(2,1) * filtDist(2,t-1) ) ...			     
                            +tra{1}(2,2) *( trc{1}(1,1) * filtDist(5,t-1) ...
					  + trc{1}(2,1) * filtDist(6,t-1) )) ...		
                 +trb(2,2)*( tra{2}(1,2) *( trc{2}(1,1) * filtDist(3,t-1) ...
					  + trc{2}(2,1) * filtDist(4,t-1) ) ...		
                            +tra{2}(2,2) *( trc{2}(1,1) * filtDist(7,t-1) ...
					  + trc{2}(2,1) * filtDist(8,t-1) ));
  predDist(8,t) = trb(1,2)*( tra{1}(1,2) *( trc{1}(1,2) * filtDist(1,t-1) ...
					  + trc{1}(2,2) * filtDist(2,t-1) ) ...			     
                            +tra{1}(2,2) *( trc{1}(1,2) * filtDist(5,t-1) ...
					  + trc{1}(2,2) * filtDist(6,t-1) )) ...		
                 +trb(2,2)*( tra{2}(1,2) *( trc{2}(1,2) * filtDist(3,t-1) ...
					  + trc{2}(2,2) * filtDist(4,t-1) ) ...		
                            +tra{2}(2,2) *( trc{2}(1,2) * filtDist(7,t-1) ...
					  + trc{2}(2,2) * filtDist(8,t-1) ));
  % 2- Update step:
  if ya(t) == 0
    if yb(t) == 0
      if yc(t) == 0 % ya=0, yb=0 and yc=0 
	filtTmp(1,t) = Oa(1,1) * Ob(1,1) * Oc(1,1) * predDist(1,t);
	filtTmp(2,t) = Oa(1,1) * Ob(1,1) * Oc(2,1) * predDist(2,t);
	filtTmp(3,t) = Oa(1,1) * Ob(2,1) * Oc(1,1) * predDist(3,t);
	filtTmp(4,t) = Oa(1,1) * Ob(2,1) * Oc(2,1) * predDist(4,t);
	filtTmp(5,t) = Oa(2,1) * Ob(1,1) * Oc(1,1) * predDist(5,t);
	filtTmp(6,t) = Oa(2,1) * Ob(1,1) * Oc(2,1) * predDist(6,t);
	filtTmp(7,t) = Oa(2,1) * Ob(2,1) * Oc(1,1) * predDist(7,t);
	filtTmp(8,t) = Oa(2,1) * Ob(2,1) * Oc(2,1) * predDist(8,t);
      else % ya=0, yb=0 and yc=1
	filtTmp(1,t) = Oa(1,1) * Ob(1,1) * Oc(1,2) * predDist(1,t);
	filtTmp(2,t) = Oa(1,1) * Ob(1,1) * Oc(2,2) * predDist(2,t);
	filtTmp(3,t) = Oa(1,1) * Ob(2,1) * Oc(1,2) * predDist(3,t);
	filtTmp(4,t) = Oa(1,1) * Ob(2,1) * Oc(2,2) * predDist(4,t);
	filtTmp(5,t) = Oa(2,1) * Ob(1,1) * Oc(1,2) * predDist(5,t);
	filtTmp(6,t) = Oa(2,1) * Ob(1,1) * Oc(2,2) * predDist(6,t);
	filtTmp(7,t) = Oa(2,1) * Ob(2,1) * Oc(1,2) * predDist(7,t);
	filtTmp(8,t) = Oa(2,1) * Ob(2,1) * Oc(2,2) * predDist(8,t);
      end;
    else  
      if yc(t) == 0 % ya=0, yb=1 and yc=0 
	filtTmp(1,t) = Oa(1,1) * Ob(1,2) * Oc(1,1) * predDist(1,t);
	filtTmp(2,t) = Oa(1,1) * Ob(1,2) * Oc(2,1) * predDist(2,t);
	filtTmp(3,t) = Oa(1,1) * Ob(2,2) * Oc(1,1) * predDist(3,t);
	filtTmp(4,t) = Oa(1,1) * Ob(2,2) * Oc(2,1) * predDist(4,t);
	filtTmp(5,t) = Oa(2,1) * Ob(1,2) * Oc(1,1) * predDist(5,t);
	filtTmp(6,t) = Oa(2,1) * Ob(1,2) * Oc(2,1) * predDist(6,t);
	filtTmp(7,t) = Oa(2,1) * Ob(2,2) * Oc(1,1) * predDist(7,t);
	filtTmp(8,t) = Oa(2,1) * Ob(2,2) * Oc(2,1) * predDist(8,t);
      else % ya=0, yb=1 and yc=1 
	filtTmp(1,t) = Oa(1,1) * Ob(1,2) * Oc(1,2) * predDist(1,t);
	filtTmp(2,t) = Oa(1,1) * Ob(1,2) * Oc(2,2) * predDist(2,t);
	filtTmp(3,t) = Oa(1,1) * Ob(2,2) * Oc(1,2) * predDist(3,t);
	filtTmp(4,t) = Oa(1,1) * Ob(2,2) * Oc(2,2) * predDist(4,t);
	filtTmp(5,t) = Oa(2,1) * Ob(1,2) * Oc(1,2) * predDist(5,t);
	filtTmp(6,t) = Oa(2,1) * Ob(1,2) * Oc(2,2) * predDist(6,t);
	filtTmp(7,t) = Oa(2,1) * Ob(2,2) * Oc(1,2) * predDist(7,t);
	filtTmp(8,t) = Oa(2,1) * Ob(2,2) * Oc(2,2) * predDist(8,t);
      end;
    end;
  else  
    if yb(t) == 0
      if yc(t) == 0 % ya=1, yb=0 and yc=0 
	filtTmp(1,t) = Oa(1,2) * Ob(1,1) * Oc(1,1) * predDist(1,t);
	filtTmp(2,t) = Oa(1,2) * Ob(1,1) * Oc(2,1) * predDist(2,t);
	filtTmp(3,t) = Oa(1,2) * Ob(2,1) * Oc(1,1) * predDist(3,t);
	filtTmp(4,t) = Oa(1,2) * Ob(2,1) * Oc(2,1) * predDist(4,t);
	filtTmp(5,t) = Oa(2,2) * Ob(1,1) * Oc(1,1) * predDist(5,t);
	filtTmp(6,t) = Oa(2,2) * Ob(1,1) * Oc(2,1) * predDist(6,t);
	filtTmp(7,t) = Oa(2,2) * Ob(2,1) * Oc(1,1) * predDist(7,t);
	filtTmp(8,t) = Oa(2,2) * Ob(2,1) * Oc(2,1) * predDist(8,t);
      else % ya=1, yb=0 and yc=1
	filtTmp(1,t) = Oa(1,2) * Ob(1,1) * Oc(1,2) * predDist(1,t);
	filtTmp(2,t) = Oa(1,2) * Ob(1,1) * Oc(2,2) * predDist(2,t);
	filtTmp(3,t) = Oa(1,2) * Ob(2,1) * Oc(1,2) * predDist(3,t);
	filtTmp(4,t) = Oa(1,2) * Ob(2,1) * Oc(2,2) * predDist(4,t);
	filtTmp(5,t) = Oa(2,2) * Ob(1,1) * Oc(1,2) * predDist(5,t);
	filtTmp(6,t) = Oa(2,2) * Ob(1,1) * Oc(2,2) * predDist(6,t);
	filtTmp(7,t) = Oa(2,2) * Ob(2,1) * Oc(1,2) * predDist(7,t);
	filtTmp(8,t) = Oa(2,2) * Ob(2,1) * Oc(2,2) * predDist(8,t);
      end;
    else  
      if yc(t) == 0 % ya=1, yb=1 and yc=0 
	filtTmp(1,t) = Oa(1,2) * Ob(1,2) * Oc(1,1) * predDist(1,t);
	filtTmp(2,t) = Oa(1,2) * Ob(1,2) * Oc(2,1) * predDist(2,t);
	filtTmp(3,t) = Oa(1,2) * Ob(2,2) * Oc(1,1) * predDist(3,t);
	filtTmp(4,t) = Oa(1,2) * Ob(2,2) * Oc(2,1) * predDist(4,t);
	filtTmp(5,t) = Oa(2,2) * Ob(1,2) * Oc(1,1) * predDist(5,t);
	filtTmp(6,t) = Oa(2,2) * Ob(1,2) * Oc(2,1) * predDist(6,t);
	filtTmp(7,t) = Oa(2,2) * Ob(2,2) * Oc(1,1) * predDist(7,t);
	filtTmp(8,t) = Oa(2,2) * Ob(2,2) * Oc(2,1) * predDist(8,t);
      else % ya=1, yb=1 and yc=1 
	filtTmp(1,t) = Oa(1,2) * Ob(1,2) * Oc(1,2) * predDist(1,t);
	filtTmp(2,t) = Oa(1,2) * Ob(1,2) * Oc(2,2) * predDist(2,t);
	filtTmp(3,t) = Oa(1,2) * Ob(2,2) * Oc(1,2) * predDist(3,t);
	filtTmp(4,t) = Oa(1,2) * Ob(2,2) * Oc(2,2) * predDist(4,t);
	filtTmp(5,t) = Oa(2,2) * Ob(1,2) * Oc(1,2) * predDist(5,t);
	filtTmp(6,t) = Oa(2,2) * Ob(1,2) * Oc(2,2) * predDist(6,t);
	filtTmp(7,t) = Oa(2,2) * Ob(2,2) * Oc(1,2) * predDist(7,t);
	filtTmp(8,t) = Oa(2,2) * Ob(2,2) * Oc(2,2) * predDist(8,t);
      end;
    end;   
  end;    
  filtDist(:,t) = filtTmp(:,t)./sum(filtTmp(:,t)); 
end;
time_hmm = toc; 
flops_hmm = flops;

% PLOT THE OPTIMAL FILTERING DISTRIBUTION:
% =======================================

filtDistPlot=[zeros(10,T)        
	      filtDist(1,:) 
              zeros(10,T) 
	      filtDist(2,:)
	      zeros(10,T) 
	      filtDist(3,:)
              zeros(10,T) 
	      filtDist(4,:)
              zeros(10,T) 
	      filtDist(5,:)
              zeros(10,T) 
	      filtDist(6,:)
              zeros(10,T) 
	      filtDist(7,:)
              zeros(10,T) 
	      filtDist(8,:)
              zeros(10,T)];     % Zero pad to make plots look nice.
figure(2)
clf;
subplot(311)
hold on
ylabel('Time (t)','fontsize',15)
%xlabel('(A_t,B_t,C_t)','fontsize',15)
zlabel('p(A_t,B_t,C_t|y_{1:t})','fontsize',15)
title('Optimal HMM filter','fontsize',15)
for t=1:5:T,
 waterfall([1:98],t,filtDistPlot(:,t)'); 
end;
view(-20,60);
rotate3d on;
arb=get(gca);
set(gca,'ygrid','off');
set(gca,'xtick',10:11:90)
set(gca,'xticklabel',{'000','001','010','011', '100','101','110','111'})


%%%%%%%%%%%% PARTICLE FILTERING BY SAMPLING FROM PRIORS %%%%%%%%%%%%%%%%
%%%%%%%%%%%% ========================================== %%%%%%%%%%%%%%%%


% INITIALISATION:
% ==============

a_pf = ones(T,N);          % These are the particles for the estimate of a.
b_pf = ones(T,N);          % These are the particles for the estimate of b.
c_pf = ones(T,N);          % These are the particles for the estimate of c.
aPred_pf = ones(T,N);      % One-step-ahead predicted values of a.
bPred_pf = ones(T,N);      % One-step-ahead predicted values of b.
cPred_pf = ones(T,N);      % One-step-ahead predicted values of c.
lik = ones(T,N);           % Likelihood.
w = ones(T,N);             % Importance weights.

% Initialise particles:
for i=1:N,
  a_pf(1,i) = rand > Pa0(1,1);  % Assume we know the initial distribution.
  b_pf(1,i) = rand > Pb0(1,1);
  c_pf(1,i) = rand > Pc0(1,1);
end;

disp(' ');
tic;                             % Initialize timer for benchmarking
flops(0);
for t=2:T,    
  fprintf('PF :  t = %i / %i  \r',t,T);
  fprintf('\n')
  
  % PREDICTION STEP:
  % ================ 
  % We use the transition priors as proposal.
  for i=1:N,
    
    if b_pf(t-1,i) == 0
      bPred_pf(t,i) = rand > trb(1,1);
      if a_pf(t-1,i) == 0
        aPred_pf(t,i) = rand > tra{1}(1,1); 
      else
        aPred_pf(t,i) = rand > tra{1}(2,1); 
      end;
      if c_pf(t-1,i) == 0
        cPred_pf(t,i) = rand > trc{1}(1,1); 
      else
        cPred_pf(t,i) = rand > trc{1}(2,1); 
      end;    
    else
      bPred_pf(t,i) = rand > trb(2,1);
      if a_pf(t-1,i) == 0
        aPred_pf(t,i) = rand > tra{2}(1,1); 
      else
        aPred_pf(t,i) = rand > tra{2}(2,1); 
      end;
      if c_pf(t-1,i) == 0
        cPred_pf(t,i) = rand > trc{2}(1,1); 
      else
        cPred_pf(t,i) = rand > trc{2}(2,1); 
      end; 
    end;     
  end;

  % EVALUATE IMPORTANCE WEIGHTS:
  % ============================
  % For our choice of proposal, the importance weights are given by:  
  for i=1:N,

  if ya(t) == 0
    if yb(t) == 0
      if yc(t) == 0 % ya=0, yb=0 and yc=0 
	if aPred_pf(t,i) == 0
	  if bPred_pf(t,i) == 0
	    if cPred_pf(t,i) == 0
	      lik(t,i) = Oa(1,1) * Ob(1,1) * Oc(1,1);
	    else
	      lik(t,i) = Oa(1,1) * Ob(1,1) * Oc(2,1);
	    end;
	  else
	    if cPred_pf(t,i) == 0
	      lik(t,i) = Oa(1,1) * Ob(2,1) * Oc(1,1);
	    else
	      lik(t,i) = Oa(1,1) * Ob(2,1) * Oc(2,1);
	    end;
	  end;
	else
	  if bPred_pf(t,i) == 0
	    if cPred_pf(t,i) == 0
	      lik(t,i) = Oa(2,1) * Ob(1,1) * Oc(1,1);
	    else
	      lik(t,i) = Oa(2,1) * Ob(1,1) * Oc(2,1);
	    end;
	  else
	    if cPred_pf(t,i) == 0
	      lik(t,i) = Oa(2,1) * Ob(2,1) * Oc(1,1);
	    else
	      lik(t,i) = Oa(2,1) * Ob(2,1) * Oc(2,1);
	    end;
	  end;
	end;  
      else % ya=0, yb=0 and yc=1
	if aPred_pf(t,i) == 0
	  if bPred_pf(t,i) == 0
	    if cPred_pf(t,i) == 0
	      lik(t,i) = Oa(1,1) * Ob(1,1) * Oc(1,2);
	    else
	      lik(t,i) = Oa(1,1) * Ob(1,1) * Oc(2,2);
	    end;
	  else
	    if cPred_pf(t,i) == 0
	      lik(t,i) = Oa(1,1) * Ob(2,1) * Oc(1,2);
	    else
	      lik(t,i) = Oa(1,1) * Ob(2,1) * Oc(2,2);
	    end;
	  end;
	else
	  if bPred_pf(t,i) == 0
	    if cPred_pf(t,i) == 0
	      lik(t,i) = Oa(2,1) * Ob(1,1) * Oc(1,2);
	    else
	      lik(t,i) = Oa(2,1) * Ob(1,1) * Oc(2,2);
	    end;
	  else
	    if cPred_pf(t,i) == 0
	      lik(t,i) = Oa(2,1) * Ob(2,1) * Oc(1,2);
	    else
	      lik(t,i) = Oa(2,1) * Ob(2,1) * Oc(2,2);
	    end;
	  end;
	end;  
      end;
    else  
      if yc(t) == 0 % ya=0, yb=1 and yc=0 
	if aPred_pf(t,i) == 0
	  if bPred_pf(t,i) == 0
	    if cPred_pf(t,i) == 0
	      lik(t,i) = Oa(1,1) * Ob(1,2) * Oc(1,1);
	    else
	      lik(t,i) = Oa(1,1) * Ob(1,2) * Oc(2,1);
	    end;
	  else
	    if cPred_pf(t,i) == 0
	      lik(t,i) = Oa(1,1) * Ob(2,2) * Oc(1,1);
	    else
	      lik(t,i) = Oa(1,1) * Ob(2,2) * Oc(2,1);
	    end;
	  end;
	else
	  if bPred_pf(t,i) == 0
	    if cPred_pf(t,i) == 0
	      lik(t,i) = Oa(2,1) * Ob(1,2) * Oc(1,1);
	    else
	      lik(t,i) = Oa(2,1) * Ob(1,2) * Oc(2,1);
	    end;
	  else
	    if cPred_pf(t,i) == 0
	      lik(t,i) = Oa(2,1) * Ob(2,2) * Oc(1,1);
	    else
	      lik(t,i) = Oa(2,1) * Ob(2,2) * Oc(2,1);
	    end;
	  end;
	end;  
      else % ya=0, yb=1 and yc=1
	if aPred_pf(t,i) == 0
	  if bPred_pf(t,i) == 0
	    if cPred_pf(t,i) == 0
	      lik(t,i) = Oa(1,1) * Ob(1,2) * Oc(1,2);
	    else
	      lik(t,i) = Oa(1,1) * Ob(1,2) * Oc(2,2);
	    end;
	  else
	    if cPred_pf(t,i) == 0
	      lik(t,i) = Oa(1,1) * Ob(2,2) * Oc(1,2);
	    else
	      lik(t,i) = Oa(1,1) * Ob(2,2) * Oc(2,2);
	    end;
	  end;
	else
	  if bPred_pf(t,i) == 0
	    if cPred_pf(t,i) == 0
	      lik(t,i) = Oa(2,1) * Ob(1,2) * Oc(1,2);
	    else
	      lik(t,i) = Oa(2,1) * Ob(1,2) * Oc(2,2);
	    end;
	  else
	    if cPred_pf(t,i) == 0
	      lik(t,i) = Oa(2,1) * Ob(2,2) * Oc(1,2);
	    else
	      lik(t,i) = Oa(2,1) * Ob(2,2) * Oc(2,2);
	    end;
	  end;
	end;  
      end;
    end;
  else  
    if yb(t) == 0
      if yc(t) == 0 % ya=1, yb=0 and yc=0 
	if aPred_pf(t,i) == 0
	  if bPred_pf(t,i) == 0
	    if cPred_pf(t,i) == 0
	      lik(t,i) = Oa(1,2) * Ob(1,1) * Oc(1,1);
	    else
	      lik(t,i) = Oa(1,2) * Ob(1,1) * Oc(2,1);
	    end;
	  else
	    if cPred_pf(t,i) == 0
	      lik(t,i) = Oa(1,2) * Ob(2,1) * Oc(1,1);
	    else
	      lik(t,i) = Oa(1,2) * Ob(2,1) * Oc(2,1);
	    end;
	  end;
	else
	  if bPred_pf(t,i) == 0
	    if cPred_pf(t,i) == 0
	      lik(t,i) = Oa(2,2) * Ob(1,1) * Oc(1,1);
	    else
	      lik(t,i) = Oa(2,2) * Ob(1,1) * Oc(2,1);
	    end;
	  else
	    if cPred_pf(t,i) == 0
	      lik(t,i) = Oa(2,2) * Ob(2,1) * Oc(1,1);
	    else
	      lik(t,i) = Oa(2,2) * Ob(2,1) * Oc(2,1);
	    end;
	  end;
	end;  
      else % ya=1, yb=0 and yc=1
	if aPred_pf(t,i) == 0
	  if bPred_pf(t,i) == 0
	    if cPred_pf(t,i) == 0
	      lik(t,i) = Oa(1,2) * Ob(1,1) * Oc(1,2);
	    else
	      lik(t,i) = Oa(1,2) * Ob(1,1) * Oc(2,2);
	    end;
	  else
	    if cPred_pf(t,i) == 0
	      lik(t,i) = Oa(1,2) * Ob(2,1) * Oc(1,2);
	    else
	      lik(t,i) = Oa(1,2) * Ob(2,1) * Oc(2,2);
	    end;
	  end;
	else
	  if bPred_pf(t,i) == 0
	    if cPred_pf(t,i) == 0
	      lik(t,i) = Oa(2,2) * Ob(1,1) * Oc(1,2);
	    else
	      lik(t,i) = Oa(2,2) * Ob(1,1) * Oc(2,2);
	    end;
	  else
	    if cPred_pf(t,i) == 0
	      lik(t,i) = Oa(2,2) * Ob(2,1) * Oc(1,2);
	    else
	      lik(t,i) = Oa(2,2) * Ob(2,1) * Oc(2,2);
	    end;
	  end;
	end;  
      end;
    else  
      if yc(t) == 0 % ya=1, yb=1 and yc=0 
	if aPred_pf(t,i) == 0
	  if bPred_pf(t,i) == 0
	    if cPred_pf(t,i) == 0
	      lik(t,i) = Oa(1,2) * Ob(1,2) * Oc(1,1);
	    else
	      lik(t,i) = Oa(1,2) * Ob(1,2) * Oc(2,1);
	    end;
	  else
	    if cPred_pf(t,i) == 0
	      lik(t,i) = Oa(1,2) * Ob(2,2) * Oc(1,1);
	    else
	      lik(t,i) = Oa(1,2) * Ob(2,2) * Oc(2,1);
	    end;
	  end;
	else
	  if bPred_pf(t,i) == 0
	    if cPred_pf(t,i) == 0
	      lik(t,i) = Oa(2,2) * Ob(1,2) * Oc(1,1);
	    else
	      lik(t,i) = Oa(2,2) * Ob(1,2) * Oc(2,1);
	    end;
	  else
	    if cPred_pf(t,i) == 0
	      lik(t,i) = Oa(2,2) * Ob(2,2) * Oc(1,1);
	    else
	      lik(t,i) = Oa(2,2) * Ob(2,2) * Oc(2,1);
	    end;
	  end;
	end;  
      else % ya=1, yb=1 and yc=1
	if aPred_pf(t,i) == 0
	  if bPred_pf(t,i) == 0
	    if cPred_pf(t,i) == 0
	      lik(t,i) = Oa(1,2) * Ob(1,2) * Oc(1,2);
	    else
	      lik(t,i) = Oa(1,2) * Ob(1,2) * Oc(2,2);
	    end;
	  else
	    if cPred_pf(t,i) == 0
	      lik(t,i) = Oa(1,2) * Ob(2,2) * Oc(1,2);
	    else
	      lik(t,i) = Oa(1,2) * Ob(2,2) * Oc(2,2);
	    end;
	  end;
	else
	  if bPred_pf(t,i) == 0
	    if cPred_pf(t,i) == 0
	      lik(t,i) = Oa(2,2) * Ob(1,2) * Oc(1,2);
	    else
	      lik(t,i) = Oa(2,2) * Ob(1,2) * Oc(2,2);
	    end;
	  else
	    if cPred_pf(t,i) == 0
	      lik(t,i) = Oa(2,2) * Ob(2,2) * Oc(1,2);
	    else
	      lik(t,i) = Oa(2,2) * Ob(2,2) * Oc(2,2);
	    end;
	  end;
	end;  
      end;
    end;
  end;    
  w(t,i) = lik(t,i);
  end;  
  w(t,:) = w(t,:)./sum(w(t,:));                % Normalise the weights.
  
  % SELECTION STEP:
  % ===============
  % Here, we give you the choice to try three different types of
  % resampling algorithms. Note that the code for these algorithms
  % applies to any problem!
  if resamplingScheme == 1
    outIndex = residualR(1:N,w(t,:)');        % Residual resampling.
  elseif resamplingScheme == 2
    outIndex = systematicR(1:N,w(t,:)');      % Systematic resampling.
  else  
    outIndex = multinomialR(1:N,w(t,:)');     % Multinomial resampling.  
  end;
  a_pf(t,:) = aPred_pf(t,outIndex);  
  b_pf(t,:) = bPred_pf(t,outIndex);  
  c_pf(t,:) = cPred_pf(t,outIndex);    
end;   % End of t loop.

time_pf = toc;    % How long did this take?
flops_pf = flops;


% PLOT THE PF JOINT FILTERING DENSITY:
% ====================================

visits = zeros(T,8);
filtDist_pf = zeros(8,T);
%filtDist_pf(:,1) = filtDist(:,1); % Assume we know initial dist.
for t=1:T,
  for i=1:N,  
    if a_pf(t,i) == 0
      if b_pf(t,i) == 0
	if c_pf(t,i) == 0
	  visits(t,1) = visits(t,1) + 1;
	else
	  visits(t,2) = visits(t,2) + 1;
	end;
      else
	if c_pf(t,i) == 0
	  visits(t,3) = visits(t,3) + 1;
	else
	  visits(t,4) = visits(t,4) + 1;
	end;
      end;
    else
      if b_pf(t,i) == 0
	if c_pf(t,i) == 0
	  visits(t,5) = visits(t,5) + 1;
	else
          visits(t,6) = visits(t,6) + 1;
     	end;
      else
	if c_pf(t,i) == 0
	  visits(t,7) = visits(t,7) + 1;
        else
	  visits(t,8) = visits(t,8) + 1;
     	end;
      end;
    end;  
  end;     
  filtDist_pf(:,t) = (visits(t,:)')./sum(visits(t,:));
end;
filtDistPlot_pf=[zeros(10,T)        
	      filtDist_pf(1,:) 
              zeros(10,T) 
	      filtDist_pf(2,:)
	      zeros(10,T) 
	      filtDist_pf(3,:)
              zeros(10,T) 
	      filtDist_pf(4,:)
              zeros(10,T) 
	      filtDist_pf(5,:)
              zeros(10,T) 
	      filtDist_pf(6,:)
              zeros(10,T) 
	      filtDist_pf(7,:)
              zeros(10,T) 
	      filtDist_pf(8,:)
              zeros(10,T)];     % Zero pad to make plots look nice.
%figure(3)
%clf;
subplot(312)
hold on
ylabel('Time (t)','fontsize',15)
%xlabel('(A_t,B_t,C_t)','fontsize',15)
zlabel('p(A_t,B_t,C_t|y_{1:t})','fontsize',15)
title('Particle filter','fontsize',15)
for t=1:5:T,
 waterfall([1:98],t,filtDistPlot_pf(:,t)'); 
end;
view(-20,60);
rotate3d on;
arb=get(gca);
set(gca,'ygrid','off');
set(gca,'xtick',10:11:90)
set(gca,'xticklabel',{'000','001','010','011', '100','101','110','111'})




%%%%%%%%%% PERFORM RAO-BLACKEWELLISED PARTICLE FILTERING %%%%%%%%%%%%%%%
%%%%%%%%%% ============================================= %%%%%%%%%%%%%%%

% INITIALISATION:
% ==============
b_rbpf = ones(T,N);        % These are the particles for the
                           % estimate of B.
pa_rbpf = ones(T,N,2);     % These are the particles for the
                           % estimate of p(A_t|y_{1:t},B_{1:t).
pc_rbpf = ones(T,N,2);     % These are the particles for the
                           % estimate of p(C_t|y_{1:t},B_{1:t}).

bPred_rbpf = ones(T,N);    % One-step-ahead predicted values of B.
paPred_rbpf = ones(T,N,2); % These are the particles for the
                           % estimate of p(A_t|y_{1:t-1},B_{1:t-1).
pcPred_rbpf = ones(T,N,2); % These are the particles for the
                           % estimate of p(C_t|y_{1:t-1},B_{1:t-1}).
paPred_rbpfR= ones(T,N,2); % These are the RESAMPLED particles for the
                           % estimate of p(A_t|y_{1:t-1},B_{1:t-1).
pcPred_rbpfR= ones(T,N,2); % These are the RESAMPLED particles for the
                           % estimate of p(C_t|y_{1:t-1},B_{1:t-1}).
lik = ones(T,N);           % Likelihood.
w = ones(T,N);             % Importance weights.

% Initialise particles:
for i=1:N,
  pa_rbpf(1,i,1) = Pa0(1,1);    % Assume we know the initial distribution.
  pa_rbpf(1,i,2) = Pa0(2,1);
  b_rbpf(1,i) = rand > Pb0(1,1);
  pc_rbpf(1,i,1) = Pc0(1,1); 
  pc_rbpf(1,i,2) = Pc0(2,1);
end;

disp(' ');

tic;                             % Initialize timer for benchmarking
flops(0);
for t=2:T,    
  fprintf('RBPF :  t = %i / %i  \r',t,T);
  fprintf('\n')
  
  % PREDICTION STEP:
  % ================ 
  % We use the transition prior as proposal.
  for i=1:N,
    if b_rbpf(t-1,i) == 0
      bPred_rbpf(t,i) = rand > trb(1,1);    
    else
      bPred_rbpf(t,i) = rand > trb(2,1);    
    end;
  end;

  % Compute predictive densities for minimum stats.
  for i=1:N,
    if b_rbpf(t-1,i) == 0
      paPred_rbpf(t,i,1) = tra{1}(1,1) * pa_rbpf(t-1,i,1) ...
			  + tra{1}(2,1) * pa_rbpf(t-1,i,2); 
      paPred_rbpf(t,i,2) = tra{1}(1,2) * pa_rbpf(t-1,i,1) ...
			  + tra{1}(2,2) * pa_rbpf(t-1,i,2); 
      pcPred_rbpf(t,i,1) = trc{1}(1,1) * pc_rbpf(t-1,i,1) ...
			  + trc{1}(2,1) * pc_rbpf(t-1,i,2); 
      pcPred_rbpf(t,i,2) = trc{1}(1,2) * pc_rbpf(t-1,i,1) ...
			  + trc{1}(2,2) * pc_rbpf(t-1,i,2); 
    else
      paPred_rbpf(t,i,1) = tra{2}(1,1) * pa_rbpf(t-1,i,1) ...
			  + tra{2}(2,1) * pa_rbpf(t-1,i,2); 
      paPred_rbpf(t,i,2) = tra{2}(1,2) * pa_rbpf(t-1,i,1) ...
			  + tra{2}(2,2) * pa_rbpf(t-1,i,2); 
      pcPred_rbpf(t,i,1) = trc{2}(1,1) * pc_rbpf(t-1,i,1) ...
			  + trc{2}(2,1) * pc_rbpf(t-1,i,2); 
      pcPred_rbpf(t,i,2) = trc{2}(1,2) * pc_rbpf(t-1,i,1) ...
			  + trc{2}(2,2) * pc_rbpf(t-1,i,2); 
    end;
  end;  
  
  % EVALUATE IMPORTANCE WEIGHTS:
  % ============================
  % For our choice of proposal, the importance weights are given by:  


    
  for i=1:N,

    if ya(t) == 0
      if yb(t) == 0
        if yc(t) == 0 % ya=0, yb=0 and yc=0 
          if bPred_rbpf(t,i) == 0
            lik(t,i) = Ob(1,1) ...
		       *(Oa(1,1)*paPred_rbpf(t,i,1)+Oa(2,1)*paPred_rbpf(t,i,2)) ...  
	               *(Oc(1,1)*pcPred_rbpf(t,i,1)+Oc(2,1)*pcPred_rbpf(t,i,2));
	  else
            lik(t,i) = Ob(2,1) ...
		       *(Oa(1,1)*paPred_rbpf(t,i,1)+Oa(2,1)*paPred_rbpf(t,i,2)) ...  
	               *(Oc(1,1)*pcPred_rbpf(t,i,1)+Oc(2,1)*pcPred_rbpf(t,i,2));
	  end;
	else   % ya=0, yb=0 and yc=1 
          if bPred_rbpf(t,i) == 0
            lik(t,i) = Ob(1,1) ...
		       *(Oa(1,1)*paPred_rbpf(t,i,1)+Oa(2,1)*paPred_rbpf(t,i,2)) ...  
	               *(Oc(1,2)*pcPred_rbpf(t,i,1)+Oc(2,2)*pcPred_rbpf(t,i,2));
	  else
            lik(t,i) = Ob(2,1) ...
		       *(Oa(1,1)*paPred_rbpf(t,i,1)+Oa(2,1)*paPred_rbpf(t,i,2)) ...  
	               *(Oc(1,2)*pcPred_rbpf(t,i,1)+Oc(2,2)*pcPred_rbpf(t,i,2));
	  end;	  
	end;    
      else	    
	if yc(t) == 0 % ya=0, yb=1 and yc=0 
          if bPred_rbpf(t,i) == 0
            lik(t,i) = Ob(1,2) ...
		       *(Oa(1,1)*paPred_rbpf(t,i,1)+Oa(2,1)*paPred_rbpf(t,i,2)) ...  
	               *(Oc(1,1)*pcPred_rbpf(t,i,1)+Oc(2,1)*pcPred_rbpf(t,i,2));
	  else
            lik(t,i) = Ob(2,2) ...
		       *(Oa(1,1)*paPred_rbpf(t,i,1)+Oa(2,1)*paPred_rbpf(t,i,2)) ...  
	               *(Oc(1,1)*pcPred_rbpf(t,i,1)+Oc(2,1)*pcPred_rbpf(t,i,2));
	  end;
	else   % ya=0, yb=1 and yc=1 
          if bPred_rbpf(t,i) == 0
            lik(t,i) = Ob(1,2) ...
		       *(Oa(1,1)*paPred_rbpf(t,i,1)+Oa(2,1)*paPred_rbpf(t,i,2)) ...  
	               *(Oc(1,2)*pcPred_rbpf(t,i,1)+Oc(2,2)*pcPred_rbpf(t,i,2));
	  else
            lik(t,i) = Ob(2,2) ...
		       *(Oa(1,1)*paPred_rbpf(t,i,1)+Oa(2,1)*paPred_rbpf(t,i,2)) ...  
	               *(Oc(1,2)*pcPred_rbpf(t,i,1)+Oc(2,2)*pcPred_rbpf(t,i,2));
	  end;	  
	end;
      end;
    else  
      if yb(t) == 0
        if yc(t) == 0 % ya=1, yb=0 and yc=0 
          if bPred_rbpf(t,i) == 0
            lik(t,i) = Ob(1,1) ...
		       *(Oa(1,2)*paPred_rbpf(t,i,1)+Oa(2,2)*paPred_rbpf(t,i,2)) ...  
	               *(Oc(1,1)*pcPred_rbpf(t,i,1)+Oc(2,1)*pcPred_rbpf(t,i,2));
	  else
            lik(t,i) = Ob(2,1) ...
		       *(Oa(1,2)*paPred_rbpf(t,i,1)+Oa(2,2)*paPred_rbpf(t,i,2)) ...  
	               *(Oc(1,1)*pcPred_rbpf(t,i,1)+Oc(2,1)*pcPred_rbpf(t,i,2));
	  end;
	else   % ya=1, yb=0 and yc=1 
          if bPred_rbpf(t,i) == 0
            lik(t,i) = Ob(1,1) ...
		       *(Oa(1,2)*paPred_rbpf(t,i,1)+Oa(2,2)*paPred_rbpf(t,i,2)) ...  
	               *(Oc(1,2)*pcPred_rbpf(t,i,1)+Oc(2,2)*pcPred_rbpf(t,i,2));
	  else
            lik(t,i) = Ob(2,1) ...
		       *(Oa(1,2)*paPred_rbpf(t,i,1)+Oa(2,2)*paPred_rbpf(t,i,2)) ...  
	               *(Oc(1,2)*pcPred_rbpf(t,i,1)+Oc(2,2)*pcPred_rbpf(t,i,2));
	  end;	  
	end;    
      else	    
	if yc(t) == 0 % ya=1, yb=1 and yc=0 
          if bPred_rbpf(t,i) == 0
            lik(t,i) = Ob(1,2) ...
		       *(Oa(1,2)*paPred_rbpf(t,i,1)+Oa(2,2)*paPred_rbpf(t,i,2)) ...  
	               *(Oc(1,1)*pcPred_rbpf(t,i,1)+Oc(2,1)*pcPred_rbpf(t,i,2));
	  else
            lik(t,i) = Ob(2,2) ...
		       *(Oa(1,2)*paPred_rbpf(t,i,1)+Oa(2,2)*paPred_rbpf(t,i,2)) ...  
	               *(Oc(1,1)*pcPred_rbpf(t,i,1)+Oc(2,1)*pcPred_rbpf(t,i,2));
	  end;
	else   % ya=1, yb=1 and yc=1 
          if bPred_rbpf(t,i) == 0
            lik(t,i) = Ob(1,2) ...
		       *(Oa(1,2)*paPred_rbpf(t,i,1)+Oa(2,2)*paPred_rbpf(t,i,2)) ...  
	               *(Oc(1,2)*pcPred_rbpf(t,i,1)+Oc(2,2)*pcPred_rbpf(t,i,2));
	  else
            lik(t,i) = Ob(2,2) ...
		       *(Oa(1,2)*paPred_rbpf(t,i,1)+Oa(2,2)*paPred_rbpf(t,i,2)) ...  
	               *(Oc(1,2)*pcPred_rbpf(t,i,1)+Oc(2,2)*pcPred_rbpf(t,i,2));
	  end;	  
	end;
      end;	    
    end;    
    w(t,i) = lik(t,i);
  end;  
  w(t,:) = w(t,:)./sum(w(t,:));                % Normalise the weights.
     

  % SELECTION STEP:
  % ===============
  % Here, we give you the choice to try three different types of
  % resampling algorithms. Note that the code for these algorithms
  % applies to any problem!
  if resamplingScheme == 1
    outIndex = residualR(1:N,w(t,:)');        % Residual resampling.
  elseif resamplingScheme == 2
    outIndex = systematicR(1:N,w(t,:)');      % Systematic resampling.
  else  
    outIndex = multinomialR(1:N,w(t,:)');     % Multinomial resampling.  
  end;
  b_rbpf(t,:) = bPred_rbpf(t,outIndex);  
  paPred_rbpfR(t,:,1) = paPred_rbpf(t,outIndex,1);
  paPred_rbpfR(t,:,2) = paPred_rbpf(t,outIndex,2);
  pcPred_rbpfR(t,:,1) = pcPred_rbpf(t,outIndex,1);
  pcPred_rbpfR(t,:,2) = pcPred_rbpf(t,outIndex,2);
     
  % COMPUTE MINIMUM STATS FOR A AND C:
  % =================================

  % Update
  for i=1:N,
    if ya(t) == 0
      if yb(t) == 0
        if yc(t) == 0 % ya=0, yb=0 and yc=0    
	    pa_rbpf(t,i,1) = Oa(1,1) * paPred_rbpfR(t,i,1);
	    pa_rbpf(t,i,2) = Oa(2,1) * paPred_rbpfR(t,i,2);
	    pc_rbpf(t,i,1) = Oc(1,1) * pcPred_rbpfR(t,i,1);
	    pc_rbpf(t,i,2) = Oc(2,1) * pcPred_rbpfR(t,i,2);
	else  % ya=0, yb=0 and yc=1     
	    pa_rbpf(t,i,1) = Oa(1,1) * paPred_rbpfR(t,i,1);
	    pa_rbpf(t,i,2) = Oa(2,1) * paPred_rbpfR(t,i,2);
	    pc_rbpf(t,i,1) = Oc(1,2) * pcPred_rbpfR(t,i,1);
	    pc_rbpf(t,i,2) = Oc(2,2) * pcPred_rbpfR(t,i,2);
  	end;
     else
        if yc(t) == 0 % ya=0, yb=1 and yc=0    
	    pa_rbpf(t,i,1) = Oa(1,1) * paPred_rbpfR(t,i,1);
	    pa_rbpf(t,i,2) = Oa(2,1) * paPred_rbpfR(t,i,2);
	    pc_rbpf(t,i,1) = Oc(1,1) * pcPred_rbpfR(t,i,1);
	    pc_rbpf(t,i,2) = Oc(2,1) * pcPred_rbpfR(t,i,2);
	else  % ya=0, yb=1 and yc=1     
	    pa_rbpf(t,i,1) = Oa(1,1) * paPred_rbpfR(t,i,1);
	    pa_rbpf(t,i,2) = Oa(2,1) * paPred_rbpfR(t,i,2);
	    pc_rbpf(t,i,1) = Oc(1,2) * pcPred_rbpfR(t,i,1);
	    pc_rbpf(t,i,2) = Oc(2,2) * pcPred_rbpfR(t,i,2);
  	end;
      end;	
    else
      if yb(t) == 0
        if yc(t) == 0 % ya=1, yb=0 and yc=0    
	    pa_rbpf(t,i,1) = Oa(1,2) * paPred_rbpfR(t,i,1);
	    pa_rbpf(t,i,2) = Oa(2,2) * paPred_rbpfR(t,i,2);
	    pc_rbpf(t,i,1) = Oc(1,1) * pcPred_rbpfR(t,i,1);
	    pc_rbpf(t,i,2) = Oc(2,1) * pcPred_rbpfR(t,i,2);
	else  % ya=1, yb=0 and yc=1     
	    pa_rbpf(t,i,1) = Oa(1,2) * paPred_rbpfR(t,i,1);
	    pa_rbpf(t,i,2) = Oa(2,2) * paPred_rbpfR(t,i,2);
	    pc_rbpf(t,i,1) = Oc(1,2) * pcPred_rbpfR(t,i,1);
	    pc_rbpf(t,i,2) = Oc(2,2) * pcPred_rbpfR(t,i,2);
  	end;
     else
        if yc(t) == 0 % y1=0, yb=1 and yc=0    
	    pa_rbpf(t,i,1) = Oa(1,2) * paPred_rbpfR(t,i,1);
	    pa_rbpf(t,i,2) = Oa(2,2) * paPred_rbpfR(t,i,2);
	    pc_rbpf(t,i,1) = Oc(1,1) * pcPred_rbpfR(t,i,1);
	    pc_rbpf(t,i,2) = Oc(2,1) * pcPred_rbpfR(t,i,2);
	else  % ya=1, yb=1 and yc=1     
	    pa_rbpf(t,i,1) = Oa(1,2) * paPred_rbpfR(t,i,1);
	    pa_rbpf(t,i,2) = Oa(2,2) * paPred_rbpfR(t,i,2);
	    pc_rbpf(t,i,1) = Oc(1,2) * pcPred_rbpfR(t,i,1);
	    pc_rbpf(t,i,2) = Oc(2,2) * pcPred_rbpfR(t,i,2);
  	end;
      end;
    end;
    pa_rbpf(t,i,:) = pa_rbpf(t,i,:)./sum(pa_rbpf(t,i,:));
    pc_rbpf(t,i,:) = pc_rbpf(t,i,:)./sum(pc_rbpf(t,i,:));    
  end;      
end;   % End of t loop.

time_rbpf = toc;    % How long did this take?
flops_rbpf = flops;




% PLOT THE PF JOINT FILTERING DENSITY:
% ====================================



afiltDist_rbpf = zeros(2,T);
bfiltDist_rbpf = zeros(2,T);
cfiltDist_rbpf = zeros(2,T);
filtDist_rbpf = zeros(8,T);
filtDist_rbpf(:,1) = filtDist(:,1); % Assume we know initial dist.
for t=1:T,
  % Marginal filtering distributions:
  bfiltDist_rbpf(1,t) = 1 - sum(b_rbpf(t,:))/N; 
  bfiltDist_rbpf(2,t) = sum(b_rbpf(t,:))/N; 
  afiltDist_rbpf(1,t) = sum(pa_rbpf(t,:,1))/N;
  afiltDist_rbpf(2,t) = sum(pa_rbpf(t,:,2))/N;
  cfiltDist_rbpf(1,t) = sum(pc_rbpf(t,:,1))/N;
  cfiltDist_rbpf(2,t) = sum(pc_rbpf(t,:,2))/N;
  % Joint filtering distribution:
  filtDist_rbpf(1,t) = afiltDist_rbpf(1,t)*bfiltDist_rbpf(1,t)*cfiltDist_rbpf(1,t);    % 000
  filtDist_rbpf(2,t) = afiltDist_rbpf(1,t)*bfiltDist_rbpf(1,t)*cfiltDist_rbpf(2,t);    % 001
  filtDist_rbpf(3,t) = afiltDist_rbpf(1,t)*bfiltDist_rbpf(2,t)*cfiltDist_rbpf(1,t);    % 010
  filtDist_rbpf(4,t) = afiltDist_rbpf(1,t)*bfiltDist_rbpf(2,t)*cfiltDist_rbpf(2,t);    % 011
  filtDist_rbpf(5,t) = afiltDist_rbpf(2,t)*bfiltDist_rbpf(1,t)*cfiltDist_rbpf(1,t);    % 100
  filtDist_rbpf(6,t) = afiltDist_rbpf(2,t)*bfiltDist_rbpf(1,t)*cfiltDist_rbpf(2,t);    % 101
  filtDist_rbpf(7,t) = afiltDist_rbpf(2,t)*bfiltDist_rbpf(2,t)*cfiltDist_rbpf(1,t);    % 110
  filtDist_rbpf(8,t) = afiltDist_rbpf(2,t)*bfiltDist_rbpf(2,t)*cfiltDist_rbpf(2,t);    % 111
end;
filtDistPlot_rbpf=[zeros(10,T)        
	      filtDist_rbpf(1,:) 
              zeros(10,T) 
	      filtDist_rbpf(2,:)
	      zeros(10,T) 
	      filtDist_rbpf(3,:)
              zeros(10,T) 
	      filtDist_rbpf(4,:)
              zeros(10,T) 
	      filtDist_rbpf(5,:)
              zeros(10,T) 
	      filtDist_rbpf(6,:)
              zeros(10,T) 
	      filtDist_rbpf(7,:)
              zeros(10,T) 
	      filtDist_rbpf(8,:)
              zeros(10,T)];     % Zero pad to make plots look nice.
%figure(4)
%clf;
subplot(313)
hold on
ylabel('Time (t)','fontsize',15)
xlabel('(A_t,B_t,C_t)','fontsize',15)
zlabel('p(A_t,B_t,C_t|y_{1:t})','fontsize',15)
title('Rao-Blackwellised particle filter','fontsize',15)
for t=1:5:T,
 waterfall([1:98],t,filtDistPlot_rbpf(:,t)'); 
end;
view(-20,60);
rotate3d on;
arb=get(gca);
set(gca,'ygrid','off');
set(gca,'xtick',10:11:90)
set(gca,'xticklabel',{'000','001','010','011', '100','101','110','111'})


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%-- CALCULATE PERFORMANCE --%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

error_pf = 0;
error_rbpf = 0;
for t=1:T,
  error_pf = error_pf + sum(abs(filtDist(:,t)-filtDist_pf(:,t))); 
  error_rbpf = error_rbpf + sum(abs(filtDist(:,t)-filtDist_rbpf(:,t))); 
end;
error_pf = error_pf/T;
error_rbpf = error_rbpf/T;

disp(' ');
disp('Time-averaged L1 errors');
disp('-----------------------');
disp(' ');
disp(['PF           = ' num2str(error_pf)]);
disp(['RBPF         = ' num2str(error_rbpf)]);

disp(' ');
disp(' ');
disp('Execution time  (seconds)');
disp('-------------------------');
disp(' ');
disp(['HMM          = ' num2str(time_hmm)]);
disp(['PF           = ' num2str(time_pf)]);
disp(['RBPF         = ' num2str(time_rbpf)]);

disp(' ');
disp(' ');
disp('Execution time  (flops)');
disp('-----------------------');
disp(' ');
disp(['HMM          = ' num2str(flops_hmm)]);
disp(['PF           = ' num2str(flops_pf)]);
disp(['RBPF         = ' num2str(flops_rbpf)]);
disp(' ');




















































